<?php

namespace Drupal\tarte_au_citron;

/**
 * Interface for the class that discover elements in js library.
 */
interface LibraryJsDiscoverInterface {

  /**
   * Check if tarte au citron files has minified versions available.
   *
   * @return bool
   *   True if minified version is available, false otherwise.
   */
  public function hasJsMinified();

  /**
   * Get the list of available services in tarteaucitron.services.js.
   *
   * @return array
   *   The array of js services available.
   */
  public function getJsServices();

  /**
   * Get the list of available texts in tarteaucitron.en.js.
   *
   * @return array
   *   The array of texts available.
   */
  public function getTextsConfig();

  /**
   * Get the tarteaucitron.js version.
   *
   * @return string
   *   The version.
   */
  public function getLibraryVersion();

  /**
   * Get the list of available configurations in tarteaucitron.js.
   *
   * @return array
   *   The array of js configurations available.
   */
  public function getJsConfig();

  /**
   * Get the list of available languages in tarteaucitron.js.
   *
   * @return array
   *   The array of available languages.
   */
  public function getTextsAvailableLanguagesConfig();

}
